﻿#include "MainWindow.h"
#include "ui_MainWindow.h"
#include <QMessageBox>
#include <QDebug>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    serial = new QSerialPort;
    serial->setPortName("COM5");
    serial->setBaudRate(115200);
    serial->setDataBits(QSerialPort::Data8);
    serial->setStopBits(QSerialPort::OneStop);
    serial->setParity(QSerialPort::NoParity);
    serial->setFlowControl(QSerialPort::NoFlowControl);
    connect(serial,SIGNAL(readyRead()),this,SLOT(slot_readyRead()));

    if(!serial->open(QIODevice::ReadWrite))
    {
        QMessageBox::warning(this,"Open serial",QString("COM5 open failed!"));
        ui->pushButton_start->setDisabled(true);
        ui->pushButton_stop->setDisabled(true);
    }
    else
    {
        ui->pushButton_start->setEnabled(true);
        ui->pushButton_stop->setDisabled(true);
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::slot_readyRead()
{
    QByteArray buf = serial->readAll();
    if(!buf.isEmpty())
    {
        char *data = buf.data();
        int ret = parse_port_data((unsigned char *)data,buf.count(),&t30monitor,&dsbusData);
        if(ret)
        {
            QString str = QString("monitor: %1 %2 %3 %4 %5 %6 %7 %8 %9 %10 %11 %12 %13 %14 %15").arg(t30monitor.t1).arg(t30monitor.t2) \
                    .arg(t30monitor.t3).arg(t30monitor.t4).arg(t30monitor.left_wheel).arg(t30monitor.left_knob).arg(t30monitor.right_knob) \
                    .arg(t30monitor.right_wheel).arg(t30monitor.switch_sa).arg(t30monitor.switch_sb).arg(t30monitor.switch_sc) \
                    .arg(t30monitor.switch_sd).arg(t30monitor.switch_se).arg(t30monitor.switch_sf).arg(t30monitor.buttons);
            qDebug() << str ;
            str = QString("sbus1: %1 %2 %3 %4 %5 %6 %7 %8 %9 %10 %11 %12 %13 %14 %15 %16").arg(dsbusData.sbus1[0]).arg(dsbusData.sbus1[1]) \
                    .arg(dsbusData.sbus1[2]).arg(dsbusData.sbus1[3]).arg(dsbusData.sbus1[4]).arg(dsbusData.sbus1[5]).arg(dsbusData.sbus1[6]) \
                    .arg(dsbusData.sbus1[7]).arg(dsbusData.sbus1[8]).arg(dsbusData.sbus1[9]).arg(dsbusData.sbus1[10]).arg(dsbusData.sbus1[11]) \
                    .arg(dsbusData.sbus1[12]).arg(dsbusData.sbus1[13]).arg(dsbusData.sbus1[14]).arg(dsbusData.sbus1[15]);
            qDebug() << str ;
            str = QString("sbus2: %1 %2 %3 %4 %5 %6 %7 %8 %9 %10 %11 %12 %13 %14 %15 %16").arg(dsbusData.sbus2[0]).arg(dsbusData.sbus2[1]) \
                    .arg(dsbusData.sbus2[2]).arg(dsbusData.sbus2[3]).arg(dsbusData.sbus2[4]).arg(dsbusData.sbus2[5]).arg(dsbusData.sbus2[6]) \
                    .arg(dsbusData.sbus2[7]).arg(dsbusData.sbus2[8]).arg(dsbusData.sbus2[9]).arg(dsbusData.sbus2[10]).arg(dsbusData.sbus2[11]) \
                    .arg(dsbusData.sbus2[12]).arg(dsbusData.sbus2[13]).arg(dsbusData.sbus2[14]).arg(dsbusData.sbus2[15]);
            qDebug() << str ;

            ui->monitor_t1->setText(QString("%1").arg(t30monitor.t1));
            ui->monitor_t2->setText(QString("%1").arg(t30monitor.t2));
            ui->monitor_t3->setText(QString("%1").arg(t30monitor.t3));
            ui->monitor_t4->setText(QString("%1").arg(t30monitor.t4));
            ui->monitor_lw->setText(QString("%1").arg(t30monitor.left_wheel));
            ui->monitor_ld->setText(QString("%1").arg(t30monitor.left_knob));
            ui->monitor_rd->setText(QString("%1").arg(t30monitor.right_knob));
            ui->monitor_rw->setText(QString("%1").arg(t30monitor.right_wheel));
            ui->monitor_sa->setText(QString("%1").arg(t30monitor.switch_sa));
            ui->monitor_sb->setText(QString("%1").arg(t30monitor.switch_sb));
            ui->monitor_sc->setText(QString("%1").arg(t30monitor.switch_sc));
            ui->monitor_sd->setText(QString("%1").arg(t30monitor.switch_sd));
            ui->monitor_se->setText(QString("%1").arg(t30monitor.switch_se));
            ui->monitor_sf->setText(QString("%1").arg(t30monitor.switch_sf));
            ui->monitor_buttons_1->setText(QString("%1").arg(t30monitor.buttons & (1) ? 1 : 0));
            ui->monitor_buttons_2->setText(QString("%1").arg(t30monitor.buttons & (1 << 1) ? 1 : 0));
            ui->monitor_buttons_3->setText(QString("%1").arg(t30monitor.buttons & (1 << 2) ? 1 : 0));
            ui->monitor_buttons_4->setText(QString("%1").arg(t30monitor.buttons & (1 << 3) ? 1 : 0));
            ui->monitor_buttons_5->setText(QString("%1").arg(t30monitor.buttons & (1 << 4) ? 1 : 0));
            ui->monitor_buttons_6->setText(QString("%1").arg(t30monitor.buttons & (1 << 5) ? 1 : 0));
            ui->monitor_buttons_7->setText(QString("%1").arg(t30monitor.buttons & (1 << 6) ? 1 : 0));
            ui->monitor_buttons_8->setText(QString("%1").arg(t30monitor.buttons & (1 << 7) ? 1 : 0));
            ui->monitor_buttons_9->setText(QString("%1").arg(t30monitor.buttons & (1 << 8) ? 1 : 0));
            ui->sbus_ch1->setText(QString("%1").arg(dsbusData.sbus1[0]));
            ui->sbus_ch2->setText(QString("%1").arg(dsbusData.sbus1[1]));
            ui->sbus_ch3->setText(QString("%1").arg(dsbusData.sbus1[2]));
            ui->sbus_ch4->setText(QString("%1").arg(dsbusData.sbus1[3]));
            ui->sbus_ch5->setText(QString("%1").arg(dsbusData.sbus1[4]));
            ui->sbus_ch6->setText(QString("%1").arg(dsbusData.sbus1[5]));
            ui->sbus_ch7->setText(QString("%1").arg(dsbusData.sbus1[6]));
            ui->sbus_ch8->setText(QString("%1").arg(dsbusData.sbus1[7]));
            ui->sbus_ch9->setText(QString("%1").arg(dsbusData.sbus1[8]));
            ui->sbus_ch10->setText(QString("%1").arg(dsbusData.sbus1[9]));
            ui->sbus_ch11->setText(QString("%1").arg(dsbusData.sbus1[10]));
            ui->sbus_ch12->setText(QString("%1").arg(dsbusData.sbus1[11]));
            ui->sbus_ch13->setText(QString("%1").arg(dsbusData.sbus1[12]));
            ui->sbus_ch14->setText(QString("%1").arg(dsbusData.sbus1[13]));
            ui->sbus_ch15->setText(QString("%1").arg(dsbusData.sbus1[14]));
            ui->sbus_ch16->setText(QString("%1").arg(dsbusData.sbus1[15]));
            ui->sbus_ch1_2->setText(QString("%1").arg(dsbusData.sbus2[0]));
            ui->sbus_ch2_2->setText(QString("%1").arg(dsbusData.sbus2[1]));
            ui->sbus_ch3_2->setText(QString("%1").arg(dsbusData.sbus2[2]));
            ui->sbus_ch4_2->setText(QString("%1").arg(dsbusData.sbus2[3]));
            ui->sbus_ch5_2->setText(QString("%1").arg(dsbusData.sbus2[4]));
            ui->sbus_ch6_2->setText(QString("%1").arg(dsbusData.sbus2[5]));
            ui->sbus_ch7_2->setText(QString("%1").arg(dsbusData.sbus2[6]));
            ui->sbus_ch8_2->setText(QString("%1").arg(dsbusData.sbus2[7]));
            ui->sbus_ch9_2->setText(QString("%1").arg(dsbusData.sbus2[8]));
            ui->sbus_ch10_2->setText(QString("%1").arg(dsbusData.sbus2[9]));
            ui->sbus_ch11_2->setText(QString("%1").arg(dsbusData.sbus2[10]));
            ui->sbus_ch12_2->setText(QString("%1").arg(dsbusData.sbus2[11]));
            ui->sbus_ch13_2->setText(QString("%1").arg(dsbusData.sbus2[12]));
            ui->sbus_ch14_2->setText(QString("%1").arg(dsbusData.sbus2[13]));
            ui->sbus_ch15_2->setText(QString("%1").arg(dsbusData.sbus2[14]));
            ui->sbus_ch16_2->setText(QString("%1").arg(dsbusData.sbus2[15]));
        }
    }
}


void MainWindow::on_pushButton_start_clicked()
{
    uint8_t cmd[10];
    request_upload_port_data(cmd,2);
    serial->write(reinterpret_cast<const char*>(&cmd),7);
    ui->pushButton_start->setDisabled(true);
    ui->pushButton_stop->setEnabled(true);
}


void MainWindow::on_pushButton_stop_clicked()
{
    uint8_t cmd[10];
    stop_upload_port_data(cmd);
    serial->write(reinterpret_cast<const char*>(&cmd),7);
    ui->pushButton_start->setEnabled(true);
    ui->pushButton_stop->setDisabled(true);
}

